@@ -16,7 +16,7 @@
 //!
 //! ```
 //! use std::io::{Cursor, Read};
-//! use bitstream_io::{BigEndian, BitReader, BitRead, ByteReader, ByteRead, LittleEndian};
+//! use bitstream_io::{BigEndian, BitReader, BitRead, ByteReader, ByteRead, FromBitStream, FromByteStream, LittleEndian};
 //!
 //! let flac: Vec<u8> = vec![0x66,0x4c,0x61,0x43,0x00,0x00,0x00,0x22,
 //!                          0x10,0x00,0x10,0x00,0x00,0x06,0x06,0x00,
@@ -47,8 +47,10 @@
 //!     block_size: u32,
 //! }
 //!
-//! impl BlockHeader {
-//!     fn read<R: std::io::Read>(r: &mut BitReader<R, BigEndian>) -> std::io::Result<Self> {
+//! impl FromBitStream for BlockHeader {
+//!     type Error = std::io::Error;
+//!
+//!     fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> std::io::Result<Self> {
 //!         Ok(Self {
 //!             last_block: r.read_bit()?,
 //!             block_type: r.read(7)?,
@@ -70,18 +72,20 @@
 //!     md5: [u8; 16],
 //! }
 //!
-//! impl Streaminfo {
-//!     fn read<R: std::io::Read>(r: &mut BitReader<R, BigEndian>) -> std::io::Result<Self> {
+//! impl FromBitStream for Streaminfo {
+//!     type Error = std::io::Error;
+//!
+//!     fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> std::io::Result<Self> {
 //!         Ok(Self {
-//!             minimum_block_size: r.read(16)?,
-//!             maximum_block_size: r.read(16)?,
+//!             minimum_block_size: r.read_to()?,
+//!             maximum_block_size: r.read_to()?,
 //!             minimum_frame_size: r.read(24)?,
 //!             maximum_frame_size: r.read(24)?,
 //!             sample_rate: r.read(20)?,
 //!             channels: r.read::<u8>(3)? + 1,
 //!             bits_per_sample: r.read::<u8>(5)? + 1,
 //!             total_samples: r.read(36)?,
-//!             md5: r.read_to_bytes()?,
+//!             md5: r.read_to()?,
 //!         })
 //!     }
 //! }
@@ -92,22 +96,23 @@
 //!     comment: Vec<String>,
 //! }
 //!
-//! impl VorbisComment {
-//!    fn read<R: std::io::Read>(
-//!        r: &mut ByteReader<R, LittleEndian>,
-//!    ) -> Result<Self, Box<dyn std::error::Error>> {
+//! impl FromBitStream for VorbisComment {
+//!    type Error = Box<dyn std::error::Error>;
 //!
-//!        fn read_entry<R: std::io::Read>(
-//!            r: &mut ByteReader<R, LittleEndian>,
+//!    fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> Result<Self, Self::Error> {
+//!        use bitstream_io::LE;
+//!
+//!        fn read_entry<R: BitRead + ?Sized>(
+//!            r: &mut R,
 //!        ) -> Result<String, Box<dyn std::error::Error>> {
 //!            use std::convert::TryInto;
-//!            let size = r.read::<u32>()?.try_into()?;
+//!            let size = r.read_as_to::<LE, u32>()?.try_into()?;
 //!            Ok(String::from_utf8(r.read_to_vec(size)?)?)
 //!        }
 //!
 //!        Ok(Self {
 //!            vendor: read_entry(r)?,
-//!            comment: (0..r.read::<u32>()?)
+//!            comment: (0..r.read_as_to::<LE, u32>()?)
 //!                .map(|_| read_entry(r))
 //!                .collect::<Result<Vec<_>, _>>()?,
 //!        })
@@ -119,17 +124,17 @@
 //! let mut reader = BitReader::endian(&mut cursor, BigEndian);
 //!
 //! // stream marker
-//! assert_eq!(&reader.read_to_bytes().unwrap(), b"fLaC");
+//! assert_eq!(&reader.read_to::<[u8; 4]>().unwrap(), b"fLaC");
 //!
 //! // metadata block header
 //! assert_eq!(
-//!     BlockHeader::read(&mut reader).unwrap(),
+//!     reader.parse::<BlockHeader>().unwrap(),
 //!     BlockHeader { last_block: false, block_type: 0, block_size: 34 }
 //! );
 //!
 //! // STREAMINFO block
 //! assert_eq!(
-//!     Streaminfo::read(&mut reader).unwrap(),
+//!     reader.parse::<Streaminfo>().unwrap(),
 //!     Streaminfo {
 //!         minimum_block_size: 4096,
 //!         maximum_block_size: 4096,
@@ -145,13 +150,13 @@
 //!
 //! // metadata block header
 //! assert_eq!(
-//!     BlockHeader::read(&mut reader).unwrap(),
+//!     reader.parse::<BlockHeader>().unwrap(),
 //!     BlockHeader { last_block: false, block_type: 4, block_size: 122 }
 //! );
 //!
-//! // VORBIS_COMMENT block (little endian)
+//! // VORBIS_COMMENT block
 //! assert_eq!(
-//!    VorbisComment::read(&mut ByteReader::new(reader.reader().unwrap())).unwrap(),
+//!    reader.parse::<VorbisComment>().unwrap(),
 //!    VorbisComment {
 //!        vendor: "reference libFLAC 1.1.4 20070213".to_string(),
 //!        comment: vec![
@@ -167,7 +172,9 @@
 
 use std::io;
 
-use super::{huffman::ReadHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, SignedNumeric};
+use super::{
+    huffman::ReadHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, Primitive, SignedNumeric,
+};
 
 /// A trait for anything that can read a variable number of
 /// potentially un-aligned values from an input stream
@@ -204,6 +211,28 @@ pub trait BitRead {
     where
         S: SignedNumeric;
 
+    /// Reads whole value from the stream whose size in bits is equal
+    /// to its type's size.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    fn read_to<V>(&mut self) -> io::Result<V>
+    where
+        V: Primitive;
+
+    /// Reads whole value from the stream whose size in bits is equal
+    /// to its type's size in an endianness that may be different
+    /// from the stream's endianness.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    fn read_as_to<F, V>(&mut self) -> io::Result<V>
+    where
+        F: Endianness,
+        V: Primitive;
+
     /// Skips the given number of bits in the stream.
     /// Since this method does not need an accumulator,
     /// it may be slightly faster than reading to an empty variable.
@@ -241,10 +270,10 @@ pub trait BitRead {
     /// # Errors
     ///
     /// Passes along any I/O error from the underlying stream.
+    #[inline(always)]
+    #[deprecated(since = "1.8.0", note = "use read_to() method instead")]
     fn read_to_bytes<const SIZE: usize>(&mut self) -> io::Result<[u8; SIZE]> {
-        let mut buf = [0; SIZE];
-        self.read_bytes(&mut buf)?;
-        Ok(buf)
+        self.read_to()
     }
 
     /// Completely fills a vector of bytes and returns it.
@@ -293,6 +322,19 @@ pub trait BitRead {
         Ok(unary)
     }
 
+    /// Parses and returns complex type
+    fn parse<F: FromBitStream>(&mut self) -> Result<F, F::Error> {
+        F::from_reader(self)
+    }
+
+    /// Parses and returns complex type with context
+    fn parse_with<'a, F: FromBitStreamWith<'a>>(
+        &mut self,
+        context: &F::Context,
+    ) -> Result<F, F::Error> {
+        F::from_reader(self, context)
+    }
+
     /// Returns true if the stream is aligned at a whole byte.
     fn byte_aligned(&self) -> bool;
 
@@ -320,7 +362,7 @@ pub trait HuffmanRead<E: Endianness> {
 /// This will read exactly as many whole bytes needed to return
 /// the requested number of bits.  It may cache up to a single partial byte
 /// but no more.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct BitReader<R: io::Read, E: Endianness> {
     reader: R,
     bitqueue: BitQueue<E, u8>,
@@ -550,6 +592,23 @@ impl<R: io::Read, E: Endianness> BitRead for BitReader<R, E> {
         E::read_signed(self, bits)
     }
 
+    #[inline]
+    fn read_to<V>(&mut self) -> io::Result<V>
+    where
+        V: Primitive,
+    {
+        E::read_primitive(self)
+    }
+
+    #[inline]
+    fn read_as_to<F, V>(&mut self) -> io::Result<V>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::read_primitive(self)
+    }
+
     /// # Examples
     /// ```
     /// use std::io::{Read, Cursor};
@@ -815,8 +874,10 @@ fn read_byte<R>(mut reader: R) -> io::Result<u8>
 where
     R: io::Read,
 {
-    let mut buf = [0; 1];
-    reader.read_exact(&mut buf).map(|()| buf[0])
+    let mut byte = 0;
+    reader
+        .read_exact(std::slice::from_mut(&mut byte))
+        .map(|()| byte)
 }
 
 fn read_aligned<R, E, N>(mut reader: R, bytes: u32, acc: &mut BitQueue<E, N>) -> io::Result<()>
@@ -932,7 +993,36 @@ pub trait ByteRead {
     /// let mut reader = ByteReader::endian(Cursor::new(&data), LittleEndian);
     /// assert_eq!(reader.read::<u16>().unwrap(), 0b1111111100000000);
     /// ```
-    fn read<N: Numeric>(&mut self) -> Result<N, io::Error>;
+    fn read<V>(&mut self) -> Result<V, io::Error>
+    where
+        V: Primitive;
+
+    /// Reads whole numeric value from stream in a potentially different endianness
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    ///
+    /// # Examples
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{BigEndian, ByteReader, ByteRead, LittleEndian};
+    /// let data = [0b00000000, 0b11111111];
+    /// let mut reader = ByteReader::endian(Cursor::new(&data), BigEndian);
+    /// assert_eq!(reader.read_as::<LittleEndian, u16>().unwrap(), 0b1111111100000000);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{BigEndian, ByteReader, ByteRead, LittleEndian};
+    /// let data = [0b00000000, 0b11111111];
+    /// let mut reader = ByteReader::endian(Cursor::new(&data), LittleEndian);
+    /// assert_eq!(reader.read_as::<BigEndian, u16>().unwrap(), 0b0000000011111111);
+    /// ```
+    fn read_as<F, V>(&mut self) -> Result<V, io::Error>
+    where
+        F: Endianness,
+        V: Primitive;
 
     /// Completely fills the given buffer with whole bytes.
     ///
@@ -951,10 +1041,10 @@ pub trait ByteRead {
     /// # Errors
     ///
     /// Passes along any I/O error from the underlying stream.
+    #[inline(always)]
+    #[deprecated(since = "1.8.0", note = "use read() method instead")]
     fn read_to_bytes<const SIZE: usize>(&mut self) -> io::Result<[u8; SIZE]> {
-        let mut buf = [0; SIZE];
-        self.read_bytes(&mut buf)?;
-        Ok(buf)
+        self.read()
     }
 
     /// Completely fills a vector of bytes and returns it.
@@ -974,11 +1064,28 @@ pub trait ByteRead {
     ///
     /// Passes along any I/O error from the underlying stream.
     fn skip(&mut self, bytes: u32) -> io::Result<()>;
+
+    /// Parses and returns complex type
+    fn parse<F: FromByteStream>(&mut self) -> Result<F, F::Error> {
+        F::from_reader(self)
+    }
+
+    /// Parses and returns complex type with context
+    fn parse_with<'a, F: FromByteStreamWith<'a>>(
+        &mut self,
+        context: &F::Context,
+    ) -> Result<F, F::Error> {
+        F::from_reader(self, context)
+    }
+
+    /// Returns mutable reference to underlying reader
+    fn reader_ref(&mut self) -> &mut dyn io::Read;
 }
 
 /// For reading aligned bytes from a stream of bytes in a given endianness.
 ///
 /// This only reads aligned values and maintains no internal state.
+#[derive(Debug)]
 pub struct ByteReader<R: io::Read, E: Endianness> {
     phantom: PhantomData<E>,
     reader: R,
@@ -1033,10 +1140,22 @@ impl<R: io::Read, E: Endianness> ByteReader<R, E> {
 
 impl<R: io::Read, E: Endianness> ByteRead for ByteReader<R, E> {
     #[inline]
-    fn read<N: Numeric>(&mut self) -> Result<N, io::Error> {
+    fn read<V>(&mut self) -> Result<V, io::Error>
+    where
+        V: Primitive,
+    {
         E::read_numeric(&mut self.reader)
     }
 
+    #[inline]
+    fn read_as<F, V>(&mut self) -> Result<V, io::Error>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::read_numeric(&mut self.reader)
+    }
+
     #[inline]
     fn read_bytes(&mut self, buf: &mut [u8]) -> io::Result<()> {
         self.reader.read_exact(buf)
@@ -1046,4 +1165,345 @@ impl<R: io::Read, E: Endianness> ByteRead for ByteReader<R, E> {
     fn skip(&mut self, bytes: u32) -> io::Result<()> {
         skip_aligned(&mut self.reader, bytes)
     }
+
+    #[inline]
+    fn reader_ref(&mut self) -> &mut dyn io::Read {
+        &mut self.reader
+    }
+}
+
+/// Implemented by complex types that don't require any additional context
+/// to parse themselves from a reader.  Analagous to `FromStr`.
+///
+/// # Example
+/// ```
+/// use std::io::{Cursor, Read};
+/// use bitstream_io::{BigEndian, BitRead, BitReader, FromBitStream};
+///
+/// #[derive(Debug, PartialEq, Eq)]
+/// struct BlockHeader {
+///     last_block: bool,
+///     block_type: u8,
+///     block_size: u32,
+/// }
+///
+/// impl FromBitStream for BlockHeader {
+///     type Error = std::io::Error;
+///
+///     fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> std::io::Result<Self> {
+///         Ok(Self {
+///             last_block: r.read_bit()?,
+///             block_type: r.read(7)?,
+///             block_size: r.read(24)?,
+///         })
+///     }
+/// }
+///
+/// let mut reader = BitReader::endian(Cursor::new(b"\x04\x00\x00\x7A"), BigEndian);
+/// assert_eq!(
+///     reader.parse::<BlockHeader>().unwrap(),
+///     BlockHeader { last_block: false, block_type: 4, block_size: 122 }
+/// );
+/// ```
+pub trait FromBitStream {
+    /// Error generated during parsing, such as `io::Error`
+    type Error;
+
+    /// Parse Self from reader
+    fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> Result<Self, Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that require some immutable context
+/// to parse themselves from a reader.
+///
+/// # Example
+/// ```
+/// use std::io::{Cursor, Read};
+/// use bitstream_io::{BigEndian, BitRead, BitReader, FromBitStreamWith};
+///
+/// #[derive(Default)]
+/// struct Streaminfo {
+///     minimum_block_size: u16,
+///     maximum_block_size: u16,
+///     minimum_frame_size: u32,
+///     maximum_frame_size: u32,
+///     sample_rate: u32,
+///     channels: u8,
+///     bits_per_sample: u8,
+///     total_samples: u64,
+///     md5: [u8; 16],
+/// }
+///
+/// #[derive(Debug, PartialEq, Eq)]
+/// struct FrameHeader {
+///     variable_block_size: bool,
+///     block_size: u32,
+///     sample_rate: u32,
+///     channel_assignment: u8,
+///     sample_size: u8,
+///     frame_number: u64,
+///     crc8: u8,
+/// }
+///
+/// impl FromBitStreamWith<'_> for FrameHeader {
+///     type Context = Streaminfo;
+///
+///     type Error = FrameHeaderError;
+///
+///     fn from_reader<R: BitRead + ?Sized>(
+///         r: &mut R,
+///         streaminfo: &Streaminfo,
+///     ) -> Result<Self, Self::Error> {
+///         if r.read::<u16>(14)? != 0b11111111111110 {
+///             return Err(FrameHeaderError::InvalidSync);
+///         }
+///
+///         if r.read_bit()? != false {
+///             return Err(FrameHeaderError::InvalidReservedBit);
+///         }
+///
+///         let variable_block_size = r.read_bit()?;
+///
+///         let block_size_bits = r.read::<u8>(4)?;
+///
+///         let sample_rate_bits = r.read::<u8>(4)?;
+///
+///         let channel_assignment = r.read::<u8>(4)?;
+///
+///         let sample_size = match r.read::<u8>(3)? {
+///             0b000 => streaminfo.bits_per_sample,
+///             0b001 => 8,
+///             0b010 => 12,
+///             0b011 => return Err(FrameHeaderError::InvalidSampleSize),
+///             0b100 => 16,
+///             0b101 => 20,
+///             0b110 => 24,
+///             0b111 => 32,
+///             _ => unreachable!(),
+///         };
+///
+///         if r.read_bit()? != false {
+///             return Err(FrameHeaderError::InvalidReservedBit);
+///         }
+///
+///         let frame_number = read_utf8(r)?;
+///
+///         Ok(FrameHeader {
+///             variable_block_size,
+///             block_size: match block_size_bits {
+///                 0b0000 => return Err(FrameHeaderError::InvalidBlockSize),
+///                 0b0001 => 192,
+///                 n @ 0b010..=0b0101 => 576 * (1 << (n - 2)),
+///                 0b0110 => r.read::<u32>(8)? + 1,
+///                 0b0111 => r.read::<u32>(16)? + 1,
+///                 n @ 0b1000..=0b1111 => 256 * (1 << (n - 8)),
+///                 _ => unreachable!(),
+///             },
+///             sample_rate: match sample_rate_bits {
+///                 0b0000 => streaminfo.sample_rate,
+///                 0b0001 => 88200,
+///                 0b0010 => 176400,
+///                 0b0011 => 192000,
+///                 0b0100 => 8000,
+///                 0b0101 => 16000,
+///                 0b0110 => 22050,
+///                 0b0111 => 24000,
+///                 0b1000 => 32000,
+///                 0b1001 => 44100,
+///                 0b1010 => 48000,
+///                 0b1011 => 96000,
+///                 0b1100 => r.read::<u32>(8)? * 1000,
+///                 0b1101 => r.read::<u32>(16)?,
+///                 0b1110 => r.read::<u32>(16)? * 10,
+///                 0b1111 => return Err(FrameHeaderError::InvalidSampleRate),
+///                 _ => unreachable!(),
+///             },
+///             channel_assignment,
+///             sample_size,
+///             frame_number,
+///             crc8: r.read(8)?
+///         })
+///     }
+/// }
+///
+/// #[derive(Debug)]
+/// enum FrameHeaderError {
+///     Io(std::io::Error),
+///     InvalidSync,
+///     InvalidReservedBit,
+///     InvalidSampleSize,
+///     InvalidBlockSize,
+///     InvalidSampleRate,
+/// }
+///
+/// impl From<std::io::Error> for FrameHeaderError {
+///     fn from(err: std::io::Error) -> Self {
+///         Self::Io(err)
+///     }
+/// }
+///
+/// fn read_utf8<R: BitRead + ?Sized>(r: &mut R) -> Result<u64, std::io::Error> {
+///     r.read(8)  // left unimplimented in this example
+/// }
+///
+/// let mut reader = BitReader::endian(Cursor::new(b"\xFF\xF8\xC9\x18\x00\xC2"), BigEndian);
+/// assert_eq!(
+///     reader.parse_with::<FrameHeader>(&Streaminfo::default()).unwrap(),
+///     FrameHeader {
+///         variable_block_size: false,
+///         block_size: 4096,
+///         sample_rate: 44100,
+///         channel_assignment: 1,
+///         sample_size: 16,
+///         frame_number: 0,
+///         crc8: 0xC2,
+///     }
+/// );
+/// ```
+///
+/// # Example with lifetime-contrained `Context`
+///
+/// In some cases, the `Context` can depend on a reference to another `struct`.
+///
+/// ```
+/// use std::io::{Cursor, Read};
+/// use bitstream_io::{BigEndian, BitRead, BitReader, FromBitStreamWith};
+///
+/// #[derive(Default)]
+/// struct ModeParameters {
+///     size_len: u8,
+///     index_len: u8,
+///     index_delta_len: u8,
+///     // ...
+/// }
+///
+/// struct AuHeaderParseContext<'a> {
+///     params: &'a ModeParameters,
+///     base_index: Option<u32>,
+/// }
+///
+/// #[derive(Debug, PartialEq, Eq)]
+/// struct AuHeader {
+///     size: u32,
+///     index: u32,
+///     // ...
+/// }
+///
+/// impl<'a> FromBitStreamWith<'a> for AuHeader {
+///     type Context = AuHeaderParseContext<'a>;
+///
+///     type Error = AuHeaderError;
+///
+///     fn from_reader<R: BitRead + ?Sized>(
+///         r: &mut R,
+///         ctx: &AuHeaderParseContext<'a>,
+///     ) -> Result<Self, Self::Error> {
+///         let size = r.read::<u32>(ctx.params.size_len as u32)?;
+///         let index = match ctx.base_index {
+///             None => r.read::<u32>(ctx.params.index_len as u32)?,
+///             Some(base_index) => {
+///                 base_index
+///                 + 1
+///                 + r.read::<u32>(ctx.params.index_delta_len as u32)?
+///             }
+///         };
+///
+///         Ok(AuHeader {
+///             size,
+///             index,
+///             // ...
+///         })
+///     }
+/// }
+///
+/// #[derive(Debug)]
+/// enum AuHeaderError {
+///     Io(std::io::Error),
+/// }
+///
+/// impl From<std::io::Error> for AuHeaderError {
+///     fn from(err: std::io::Error) -> Self {
+///         Self::Io(err)
+///     }
+/// }
+///
+/// let mut reader = BitReader::endian(Cursor::new(b"\xFF\xEA\xFF\x10"), BigEndian);
+///
+/// let mode_params = ModeParameters {
+///     size_len: 10,
+///     index_len: 6,
+///     index_delta_len: 2,
+///     // ...
+/// };
+///
+/// let mut ctx = AuHeaderParseContext {
+///     params: &mode_params,
+///     base_index: None,
+/// };
+///
+/// let header1 = reader.parse_with::<AuHeader>(&ctx).unwrap();
+/// assert_eq!(
+///     header1,
+///     AuHeader {
+///         size: 1023,
+///         index: 42,
+///     }
+/// );
+///
+/// ctx.base_index = Some(header1.index);
+///
+/// assert_eq!(
+///     reader.parse_with::<AuHeader>(&ctx).unwrap(),
+///     AuHeader {
+///         size: 1020,
+///         index: 44,
+///     }
+/// );
+/// ```
+pub trait FromBitStreamWith<'a> {
+    /// Some context to use when parsing
+    type Context: 'a;
+
+    /// Error generated during parsing, such as `io::Error`
+    type Error;
+
+    /// Parse Self from reader with the given context
+    fn from_reader<R: BitRead + ?Sized>(
+        r: &mut R,
+        context: &Self::Context,
+    ) -> Result<Self, Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that don't require any additional context
+/// to parse themselves from a reader.  Analagous to `FromStr`.
+pub trait FromByteStream {
+    /// Error generated during parsing, such as `io::Error`
+    type Error;
+
+    /// Parse Self from reader
+    fn from_reader<R: ByteRead + ?Sized>(r: &mut R) -> Result<Self, Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that require some additional context
+/// to parse themselves from a reader.  Analagous to `FromStr`.
+pub trait FromByteStreamWith<'a> {
+    /// Some context to use when parsing
+    type Context: 'a;
+
+    /// Error generated during parsing, such as `io::Error`
+    type Error;
+
+    /// Parse Self from reader
+    fn from_reader<R: ByteRead + ?Sized>(
+        r: &mut R,
+        context: &Self::Context,
+    ) -> Result<Self, Self::Error>
+    where
+        Self: Sized;
 }
