@@ -17,7 +17,7 @@
 //! ```
 //! use std::convert::TryInto;
 //! use std::io::Write;
-//! use bitstream_io::{BigEndian, BitWriter, BitWrite, ByteWriter, ByteWrite, LittleEndian};
+//! use bitstream_io::{BigEndian, BitWriter, BitWrite, ByteWriter, ByteWrite, LittleEndian, ToBitStream};
 //!
 //! #[derive(Debug, PartialEq, Eq)]
 //! struct BlockHeader {
@@ -26,8 +26,10 @@
 //!     block_size: u32,
 //! }
 //!
-//! impl BlockHeader {
-//!     fn write<W: std::io::Write>(&self, w: &mut BitWriter<W, BigEndian>) -> std::io::Result<()> {
+//! impl ToBitStream for BlockHeader {
+//!     type Error = std::io::Error;
+//!
+//!     fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> std::io::Result<()> {
 //!         w.write_bit(self.last_block)?;
 //!         w.write(7, self.block_type)?;
 //!         w.write(24, self.block_size)
@@ -47,10 +49,12 @@
 //!     md5: [u8; 16],
 //! }
 //!
-//! impl Streaminfo {
-//!     fn write<W: std::io::Write>(&self, w: &mut BitWriter<W, BigEndian>) -> std::io::Result<()> {
-//!         w.write(16, self.minimum_block_size)?;
-//!         w.write(16, self.maximum_block_size)?;
+//! impl ToBitStream for Streaminfo {
+//!     type Error = std::io::Error;
+//!
+//!     fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> std::io::Result<()> {
+//!         w.write_from(self.minimum_block_size)?;
+//!         w.write_from(self.maximum_block_size)?;
 //!         w.write(24, self.minimum_frame_size)?;
 //!         w.write(24, self.maximum_frame_size)?;
 //!         w.write(20, self.sample_rate)?;
@@ -97,10 +101,10 @@
 //! writer.write_bytes(b"fLaC").unwrap();
 //!
 //! // metadata block header
-//! (BlockHeader { last_block: false, block_type: 0, block_size: 34 }).write(&mut writer).unwrap();
+//! writer.build(&BlockHeader { last_block: false, block_type: 0, block_size: 34 }).unwrap();
 //!
 //! // STREAMINFO block
-//! (Streaminfo {
+//! writer.build(&Streaminfo {
 //!     minimum_block_size: 4096,
 //!     maximum_block_size: 4096,
 //!     minimum_frame_size: 1542,
@@ -110,7 +114,7 @@
 //!     bits_per_sample: 16,
 //!     total_samples: 304844,
 //!     md5: *b"\xFA\xF2\x69\x2F\xFD\xEC\x2D\x5B\x30\x01\x76\xB4\x62\x88\x7D\x92",
-//! }).write(&mut writer).unwrap();
+//! }).unwrap();
 //!
 //! let comment = VorbisComment {
 //!     vendor: "reference libFLAC 1.1.4 20070213".to_string(),
@@ -123,11 +127,13 @@
 //! };
 //!
 //! // metadata block header
-//! (BlockHeader {
-//!    last_block: false,
-//!    block_type: 4,
-//!    block_size: comment.len().try_into().unwrap(),
-//! }).write(&mut writer).unwrap();
+//! writer.build(
+//!     &BlockHeader {
+//!         last_block: false,
+//!         block_type: 4,
+//!         block_size: comment.len().try_into().unwrap(),
+//!     }
+//! ).unwrap();
 //!
 //! // VORBIS_COMMENT block (little endian)
 //! comment.write(&mut ByteWriter::new(writer.writer().unwrap())).unwrap();
@@ -161,7 +167,9 @@ use std::convert::From;
 use std::io;
 use std::ops::{AddAssign, Rem};
 
-use super::{huffman::WriteHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, SignedNumeric};
+use super::{
+    huffman::WriteHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, Primitive, SignedNumeric,
+};
 
 /// For writing bit values to an underlying stream in a given endianness.
 ///
@@ -322,6 +330,28 @@ pub trait BitWrite {
     where
         S: SignedNumeric;
 
+    /// Writes whole value to the stream whose size in bits
+    /// is equal to its type's size.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    fn write_from<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive;
+
+    /// Writes whole value to the stream whose size in bits
+    /// is equal to its type's size in an endianness that may
+    /// be different from the stream's endianness.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    fn write_as_from<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive;
+
     /// Writes the entirety of a byte buffer to the stream.
     ///
     /// # Errors
@@ -439,6 +469,20 @@ pub trait BitWrite {
         }
     }
 
+    /// Builds and writes complex type
+    fn build<T: ToBitStream>(&mut self, build: &T) -> Result<(), T::Error> {
+        build.to_writer(self)
+    }
+
+    /// Builds and writes complex type with context
+    fn build_with<'a, T: ToBitStreamWith<'a>>(
+        &mut self,
+        build: &T,
+        context: &T::Context,
+    ) -> Result<(), T::Error> {
+        build.to_writer(self, context)
+    }
+
     /// Returns true if the stream is aligned at a whole byte.
     fn byte_aligned(&self) -> bool;
 
@@ -606,6 +650,23 @@ impl<W: io::Write, E: Endianness> BitWrite for BitWriter<W, E> {
         E::write_signed(self, bits, value)
     }
 
+    #[inline]
+    fn write_from<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive,
+    {
+        E::write_primitive(self, value)
+    }
+
+    #[inline]
+    fn write_as_from<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::write_primitive(self, value)
+    }
+
     #[inline]
     fn write_bytes(&mut self, buf: &[u8]) -> io::Result<()> {
         if self.byte_aligned() {
@@ -733,6 +794,23 @@ where
         E::write_signed(self, bits, value)
     }
 
+    #[inline]
+    fn write_from<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive,
+    {
+        E::write_primitive(self, value)
+    }
+
+    #[inline]
+    fn write_as_from<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::write_primitive(self, value)
+    }
+
     #[inline]
     fn write_unary1(&mut self, value: u32) -> io::Result<()> {
         self.bits += (value + 1).into();
@@ -922,7 +1000,7 @@ impl<N: Default + Copy, E: Endianness> BitRecorder<N, E> {
         }
     }
 
-    /// Creates new recorder with the given endiannness
+    /// Creates new recorder with the given endianness
     #[inline]
     pub fn endian(_endian: E) -> Self {
         BitRecorder {
@@ -983,6 +1061,23 @@ where
         Ok(())
     }
 
+    #[inline]
+    fn write_from<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive,
+    {
+        E::write_primitive(self, value)
+    }
+
+    #[inline]
+    fn write_as_from<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::write_primitive(self, value)
+    }
+
     #[inline]
     fn write_unary0(&mut self, value: u32) -> io::Result<()> {
         self.records.push(WriteRecord::Unary0(value));
@@ -1027,8 +1122,7 @@ fn write_byte<W>(mut writer: W, byte: u8) -> io::Result<()>
 where
     W: io::Write,
 {
-    let buf = [byte];
-    writer.write_all(&buf)
+    writer.write_all(std::slice::from_ref(&byte))
 }
 
 fn write_unaligned<W, E, N>(
@@ -1153,7 +1247,36 @@ pub trait ByteWrite {
     /// writer.write(0b0000000011111111u16).unwrap();
     /// assert_eq!(writer.into_writer(), [0b11111111, 0b00000000]);
     /// ```
-    fn write<N: Numeric>(&mut self, value: N) -> io::Result<()>;
+    fn write<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive;
+
+    /// Writes whole numeric value to stream in a potentially different endianness
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    ///
+    /// # Examples
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{BigEndian, ByteWriter, ByteWrite, LittleEndian};
+    /// let mut writer = ByteWriter::endian(Vec::new(), BigEndian);
+    /// writer.write_as::<LittleEndian, u16>(0b0000000011111111).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b11111111, 0b00000000]);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{BigEndian, ByteWriter, ByteWrite, LittleEndian};
+    /// let mut writer = ByteWriter::endian(Vec::new(), LittleEndian);
+    /// writer.write_as::<BigEndian, u16>(0b0000000011111111).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b00000000, 0b11111111]);
+    /// ```
+    fn write_as<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive;
 
     /// Writes the entirety of a byte buffer to the stream.
     ///
@@ -1161,16 +1284,140 @@ pub trait ByteWrite {
     ///
     /// Passes along any I/O error from the underlying stream.
     fn write_bytes(&mut self, buf: &[u8]) -> io::Result<()>;
+
+    /// Builds and writes complex type
+    fn build<T: ToByteStream>(&mut self, build: &T) -> Result<(), T::Error> {
+        build.to_writer(self)
+    }
+
+    /// Builds and writes complex type with context
+    fn build_with<'a, T: ToByteStreamWith<'a>>(
+        &mut self,
+        build: &T,
+        context: &T::Context,
+    ) -> Result<(), T::Error> {
+        build.to_writer(self, context)
+    }
+
+    /// Returns mutable reference to underlying writer
+    fn writer_ref(&mut self) -> &mut dyn io::Write;
 }
 
 impl<W: io::Write, E: Endianness> ByteWrite for ByteWriter<W, E> {
     #[inline]
-    fn write<N: Numeric>(&mut self, value: N) -> io::Result<()> {
+    fn write<V>(&mut self, value: V) -> io::Result<()>
+    where
+        V: Primitive,
+    {
         E::write_numeric(&mut self.writer, value)
     }
 
+    #[inline]
+    fn write_as<F, V>(&mut self, value: V) -> io::Result<()>
+    where
+        F: Endianness,
+        V: Primitive,
+    {
+        F::write_numeric(&mut self.writer, value)
+    }
+
     #[inline]
     fn write_bytes(&mut self, buf: &[u8]) -> io::Result<()> {
         self.writer.write_all(buf)
     }
+
+    #[inline]
+    fn writer_ref(&mut self) -> &mut dyn io::Write {
+        &mut self.writer
+    }
+}
+
+/// Implemented by complex types that don't require any additional context
+/// to build themselves to a writer
+///
+/// # Example
+/// ```
+/// use std::io::{Cursor, Read};
+/// use bitstream_io::{BigEndian, BitWrite, BitWriter, ToBitStream};
+///
+/// #[derive(Debug, PartialEq, Eq)]
+/// struct BlockHeader {
+///     last_block: bool,
+///     block_type: u8,
+///     block_size: u32,
+/// }
+///
+/// impl ToBitStream for BlockHeader {
+///     type Error = std::io::Error;
+///
+///     fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> std::io::Result<()> {
+///         w.write_bit(self.last_block)?;
+///         w.write(7, self.block_type)?;
+///         w.write(24, self.block_size)
+///     }
+/// }
+///
+/// let mut data = Vec::new();
+/// let mut writer = BitWriter::endian(&mut data, BigEndian);
+/// writer.build(&BlockHeader { last_block: false, block_type: 4, block_size: 122 }).unwrap();
+/// assert_eq!(data, b"\x04\x00\x00\x7A");
+/// ```
+pub trait ToBitStream {
+    /// Error generated during building, such as `io::Error`
+    type Error;
+
+    /// Generate self to writer
+    fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> Result<(), Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that require additional context
+/// to build themselves to a writer
+pub trait ToBitStreamWith<'a> {
+    /// Some context to use when writing
+    type Context: 'a;
+
+    /// Error generated during building, such as `io::Error`
+    type Error;
+
+    /// Generate self to writer
+    fn to_writer<W: BitWrite + ?Sized>(
+        &self,
+        w: &mut W,
+        context: &Self::Context,
+    ) -> Result<(), Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that don't require any additional context
+/// to build themselves to a writer
+pub trait ToByteStream {
+    /// Error generated during building, such as `io::Error`
+    type Error;
+
+    /// Generate self to writer
+    fn to_writer<W: ByteWrite + ?Sized>(&self, w: &mut W) -> Result<(), Self::Error>
+    where
+        Self: Sized;
+}
+
+/// Implemented by complex types that require additional context
+/// to build themselves to a writer
+pub trait ToByteStreamWith<'a> {
+    /// Some context to use when writing
+    type Context: 'a;
+
+    /// Error generated during building, such as `io::Error`
+    type Error;
+
+    /// Generate self to writer
+    fn to_writer<W: ByteWrite + ?Sized>(
+        &self,
+        w: &mut W,
+        context: &Self::Context,
+    ) -> Result<(), Self::Error>
+    where
+        Self: Sized;
 }
